#include <dos.h>
#include <dir.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <process.h>
#include "k3eeprog.h"

void main(int argc, char *argv[])
     {
      char choice; // user selection from the program's main window


      // Turn off the cursor.
      _setcursortype(_NOCURSOR);

      // Evaluate the user commandline entries.
      if(system_setup(argc, argv[1], argv[2]))
	 exit(0);

      // Check to see if EVS05 or MMDS05, and 00014VXX.MEM are present in the
      // current directory. If either one of the files is missing, exit to DOS.
      // Remove any residual K3EEPROG files if they are present.
      if(files_present())
	 exit(0);

      // Display program opening screen.
      main_screen();

      // Display user instruction box.
      user_instruction_box();

      // Rename the user-created STARTUP.05 to a temporary new name.
      rename("STARTUP.05","STARTUP.SAV");

      // Display the user menu. Continue to process the user's commands until
      // the EXIT command is selected.
      do
	{
	 switch(choice = user_selection())
	    {
	     case '1':
	       clrscr();
	       program_eeprom(argv[2]);
	     break;
	     case '2':
	       clrscr();
	       read_eeprom(argv[2]);
	     break;
	     case '3':
	     break;
	     default:
		invalid_choice();
	     break;
	    }

	 // Clear the screen.
	 clrscr();
	 fflush(stdin);
	}while(choice != '3');

      // Restore the user-created STARTUP.05 to its original name.
      rename("STARTUP.SAV","STARTUP.05");

      // Clear screen.
      clrscr();

      exit(0);
     }

/*                        user_selection function
*
* Function input variables: None.
*
* Function outputs: an integer.
*                   1: The user has selected to program the Personality EEPROM.
*                   2: The user has selected to read the Personality EEPROM.
*                   3: The user has selected to exit the program.
*
*/

char user_selection(void)
    {
     char selection;

     printf("1) Program Personality EEPROM\n");
     printf("2) Read Personality EEPROM Bytes\n");
     printf("3) Quit\n");
     printf("\n\n");
     printf("Enter a selection: ");

     selection = getchar();
     return (selection);

    }

/*                         program_eeprom function
*
* Function input variables: char port[]
*
* Function outputs: None.
*
* This function programs the Personality EEPROM of the MMDS05's resident
* device.
*
*/
void program_eeprom(char port[])
     {
      int startup_file_present; // flag indicating that a MMDS05 startup file
      int flag;				// is present
      int return_flag; //generic flag

      // Get the data to be programmed into the K3 from the user.
      // After the data has been entered, have the user verify it.
      // If the data is not correct, have the user re-enter the data.
      do
	{
	 enter_data();
	 return_flag = data_verification_box(program);
	 if(return_flag == 1)
	    return;
	}while(return_flag == -1);

      // Construct a temporary STARTUP.05 file to program the device.
      // If the file cannot be opened, display an error box and
      // proceed with a program shut down.
      if(program_startup_file())
	{
	 // Display memory error screen.
	 memory_error_box();

	 // Restore the user-created STARTUP.05 to its original name.
	 rename("STARTUP.SAV", "STARTUP.05");

	 // Exit to DOS.
	 exit(0);
	}

      // Call either MMDS05 or EVS05 and wait for the result.
      if(emulator_type)
	 spawnl(P_WAIT, "evs05.exe","evs05", port, NULL);
      else
	 spawnl(P_WAIT, "mmds05.exe", "mmds05",port, NULL);

      // Turn the off the cursor.
      _setcursortype(_NOCURSOR);

      // Delete the S-record programming file.
      remove("K3PROG.S19");

      // Delete the startup file.
      remove("STARTUP.05");


      // Evaluate the results of the emulator's execution.
      // If the results are good, display a message box and exit to DOS.
      // Otherwise display an appropriate error box and exit to DOS.
      switch(evaluate_program_results())
	    {
	     case -1:
		emulator_error_box();
		break;
	     case 0:
		programmed_box();
		break;
	     case 1:
		eeprom_error_box();
		break;
	     default:
		break;
	    }

	// Remove the MMDS05 log file.
	remove("K3EEPROG.LOG");

	return;
     }

/*                           read_eeprom function
*
* Function input variables: char port[]
*
* Function outputs: None.
*
* This function read the Personality EEPROM of the MMDS05's resident
* device.
*
*/
void read_eeprom(char port[])
     {
      int startup_file_present; // flag indicating that a MMDS05/EVS05 startup file
      int flag;				// is present
      int return_flag; //generic flag


      // Construct a temporary STARTUP.05 file to read the device.
      // If the file cannot be opened, display an error box and
      // proceed with a program shut down.
      if(read_startup_file())
	{
	 // Display memory error screen.
	 memory_error_box();

	 // If a user startup file has been renamed, restore its proper name.
	 rename("STARTUP.SAV", "STARTUP.05");

	 // Exit to DOS.
	 exit(0);
	}

      // Call either MMDS05 or EVS05 and wait for the result.
      if(emulator_type)
	 spawnl(P_WAIT, "evs05.exe","evs05", port, NULL);
      else
	 spawnl(P_WAIT, "mmds05.exe", "mmds05",port, NULL);

      // Turn the off the cursor.
      _setcursortype(_NOCURSOR);

      // Delete the S-record reading algorithm.
      remove("K3READ.S19");

      // Delete the startup file.
      remove("STARTUP.05");

      // Evaluate the results of the emulator's execution.
      // If the results are good, display a message box and exit to DOS.
      // Otherwise display an appropriate error box and exit to DOS.
      switch(evaluate_read_results())
	    {
	     case -1:
		emulator_error_box();
		break;
	     case 0:
		data_verification_box(read);
		break;
	     case 1:
		emulator_error_box();
		break;
	     default:
		break;
	    }

	 // Remove the MMDS05 log file.
	 remove("K3EEPROG.LOG");

	return;
     }

/*                         system_setup function
*
* Function input variables: int commandline_number;
*                           char commandline_emulator[];
*                           char commandline_port[];
*
* Function outputs: an integer;
*                   0: If the system setup was successful.
*                   1: If the system setup failed.
*
* This function determines emulation system and the COM port will be used by
* K3EEPROG.EXE.
*
*/

int system_setup(int commandline_number, char commandline_emulator[],
		 char commandline_port[])
     {
      int flag=0; // function return flag
      int port;
      // Check the number of commandline parameters to make sure that two or
      // three parameters are present.
      if((commandline_number < 2) || (commandline_number > 3))
	{
	  commandline_error_box();
	  return(1);
	}

      // Check the emulator type parameter and set the emulator_type flag
      // appropriately.
      if(!strcmpi("MMDS05", commandline_emulator))
	 emulator_type = 0;

      else if(!strcmpi("EVS05", commandline_emulator))
	      emulator_type = 1;
      else
	  {
	   commandline_error_box();
	   return(1);
	  }

      // Check the COM port commandline parameter.
      if(commandline_number == 2)
	 port = 1;
      else
	 port = atoi(commandline_port);
      if(port<1 || port > 4)
	  {
	   commandline_error_box();
	   flag=1;
	  }

       return(flag);
     }

/*                        enter_data function
*
* Function input variables: None.
*
* Function outputs: None.
*
*
* This function accepts data from the user to be programmed into the
* K3's personality EEPROM.
*
*/

void enter_data(void)
     {
      int i,j;
      char c;

      // Let the user enter the data to be programmed into the K3.
      for(i=0;i<16;i++)
	 {
	  data[i][3] = '\0';
	  printf("Enter the data for bits %3.3d-%3.3d: ", (i*8), ((i+1)*8)-1);
	  j=0;
	  while(j<2)
	     {
	      c = getch();
	      // Check to see if the user entered a valid hexadecimal number.
	      // If not, beep and do not accept the character.
	      if(not_hex_number(c))
		 beep();
	      else
		{
		 data[i] [j] = c;
		 putchar(c);
		 j++;
		}
	     }
	  printf("\n");
	 }

      // Convert all user entered data to upper case.
      for(i=0;i<16;i++)
	  strupr(data[i]);
      return;
     }

/*                     program_startup_file function
*
* Function input variables: None.
*
* Function outputs: an integer;
*                   0: If the startup file was successfully opened.
*                   1: If the startup file failed to open.
*
*
* This function constructs a temporary startup file to control emulator
* operation so that it can program the PEEPROM.
*
*/

int program_startup_file(void)
    {
     int i; // generic counter variable
     FILE *startup_file;  // temporary STARTUP.05 file pointer
     FILE *programming_file; // temporary S-record file to program PEEPROM.

     // Create a temporary S-record file to program PEEPROM.
     // If the file fails to open return a one.
     if((programming_file = fopen("K3PROG.S19","w")) == NULL)
	 return(1);

     // Incorporate the S-record data into the file.
     for(i=0;i<9;i++)
     fprintf(programming_file,"%s\n",S19_data[i]);

     // Close temporary S-record programming file.
     fclose(programming_file);


     // Open temporary startup file. If it fails to open, return a one.
     if((startup_file = fopen("startup.05","w")) == NULL)
	 return(1);

     // Incorporate the user data in the startup file.
     for(i=0;i<16;i++)
     fprintf(startup_file,"mm %02X %s\n", (0xC0+i), data[i]);

     // Commands to execute the S19 file.
     fprintf(startup_file,"load k3prog.s19\n");
     fprintf(startup_file,"reset\n");
     fprintf(startup_file,"br 13D\n");
     fprintf(startup_file,"br 13F\n");
     fprintf(startup_file,"g\n");
     fprintf(startup_file,"wait 2\n");

     // Commands to open a MMDS05 log file.
     fprintf(startup_file,"stop\n");
     fprintf(startup_file,"lf k3eeprog.log\n");
     fprintf(startup_file,"reg\n");
     fprintf(startup_file,"lf\n");
     fprintf(startup_file,"exit");

     // Close startup file.
     fclose(startup_file);

     return(0);
    }

/*                        read_startup_file function
*
* Function input variables: None.
*
* Function outputs: an integer;
*                   0: If the startup file was successfully opened.
*                   1: If the startup file failed to open.
*
*
* This function constructs a temporary startup file to control emulator
* operation.
*
*/

int read_startup_file(void)
    {
     int i; // generic counter variable
     FILE *startup_file;  // temporary STARTUP.05 file pointer
     FILE *read_file; // temporary S-record file to read PEEPROM.

     // Create a temporary S-record file to read PEEPROM.
     // If the file fails to open, return a one.
     if((read_file = fopen("K3READ.S19","w")) == NULL)
	 return(1);

     // Incorporate the S-record data into the file.
     for(i=0;i<4;i++)
     fprintf(read_file,"%s\n",read_data[i]);

     // Close temporary S-record programming file.
     fclose(read_file);

     // Open startup file. If it fails to open, return a one.
     if((startup_file = fopen("startup.05","w")) == NULL)
	 return(1);

     // Commands to execute the S19 file.
     fprintf(startup_file,"load K3READ.S19\n");
     fprintf(startup_file,"reset\n");
     fprintf(startup_file,"wait 1\n");
     fprintf(startup_file,"br $11B\n");
     fprintf(startup_file,"g\n");
     fprintf(startup_file,"wait 2\n");

     // Commands to open a MMDS05 log file.
     fprintf(startup_file,"stop\n");
     fprintf(startup_file,"wait 1\n");
     fprintf(startup_file,"lf k3eeprog.log\n");
     fprintf(startup_file,"reg\n");
     fprintf(startup_file,"md c0\n");
     fprintf(startup_file,"lf\n");
     fprintf(startup_file,"exit");

     // Close startup file.
     fclose(startup_file);

     return(0);
    }

/*                          files_present function
*
* Function input variables: None.
*
* Function outputs: an integer;
*                   0: all the necessary files are present in the current directory.
*                   1: one or more of the necessary files are missing.
*
* This function checks for either EVS05 or MMDS05, and 00014VXX.MEM. These
* files are necessary for this program to operate. Remove any residual files that may
* be left over if a previous running of K3EEPROG was unsuccessful.
*
*/
int files_present(void)
    {
     struct find_t fb;  // structure for the _dos_findfirst() function
     int flag = 0; // function return flag

     // Check to see if either MMDS05 or EVS05 are present in the current
     // directory. If neither are, display an error box and exit to DOS.
     if(emulator_type)
       {
	if(_dos_findfirst("EVS05.EXE", _A_NORMAL, &fb))
	  {
	   no_evs05();
	   flag = 1;
	  }
       }
     else
       {
	 if(_dos_findfirst("MMDS05.EXE", _A_NORMAL, &fb))
	   {
	    no_mmds05();
	    flag = 1;
	   }
       }

     // Check to see if the K3 personality file (00014XXX.MEM)
     // is present in the current directory.
     // If it isn't display an error box and exit to DOS.
     if(_dos_findfirst("00014*.MEM", _A_NORMAL, &fb))
       {
	no_personality_file();
	flag = 1;
       }

     // If the program's log file is present, remove it.
     if(!_dos_findfirst("K3EEPROG.LOG", _A_NORMAL, &fb))
	remove("K3EEPROG.LOG");


     // If the PEEPROM's programming algorithm is present, remove it.
     if(!_dos_findfirst("K3PROG.S19", _A_NORMAL, &fb))
	remove("K3PROG.S19");

     // If the PEEPROM's read algorithm is present, remove it.
     if(!_dos_findfirst("K3READ.S19", _A_NORMAL, &fb))
	remove("K3READ.S19");

     return(flag);
    }



/*                     evaluate_results_program function
*
* Function input variables: None.
*
* Function outputs: an integer;
*                  -1: If the emulator log file failed to open.
*                   0: If the K3's personality EEPROM was successfully
*                      programmed.
*                   1: If the K3's personality EEPROM failed to program.
*
* This function reads the log file made by the emulator and from the address
* of the PC register, determines if the EEPROM was successfully programmed or
* not. The addresses are as follows: $013D-EEPROM was correctly programmed,
* $013F-EEPROM failed to be programmed.
*
*/

int evaluate_program_results(void)
    {
     FILE *k3eeprog_log_file;  // file pointer to emulator log file.
     char linestring[80];  // generic character string.


     // Open emulator log file. If it fails to open, return a negative one.
     if((k3eeprog_log_file = fopen("k3eeprog.log","r")) == NULL)
	 return(-1);

     // Find the line in the log file that contain the register contents.
     // When it is found, determine the location of the PC register.
     while(fgets(linestring,80,k3eeprog_log_file) != NULL)
	  {
	   if(strstr(linestring,"PC") != NULL)
	     {
	      // If the PC address is $13D return a zero.
	      if(strstr(linestring,"13D") != NULL)
		 return(0);

	      // If the PC address is $13F return a one.
	      else if(strstr(linestring,"13F") != NULL)
		 return(1);

	      // If neither address is found, return a negative one.
	      else
		 return(-1);
	     }
	  }

     return(-1);
    }

/*                     evaluate_read_results function
*
* Function input variables: None.
*
* Function outputs: an integer;
*                  -1: If the emulator log file failed to open.
*                   0: If the K3's personality EEPROM was successfully
*                      read.
*                   1: If the K3's personality EEPROM failed to be read.
*
* This function reads the log file made by the emulator and from the address
* of the PC register, determines if the EEPROM was successfully read or
* not. The addresses are as follows: $011B-EEPROM was correctly read,
* any other address- EEPROM failed to be read.
*
*/

int evaluate_read_results(void)
    {
     FILE *k3eeprog_log_file;  // file pointer to emulator log file.
     char linestring[80];  // generic character string.
     int i; // counter variable


     // Open emulator log file. If it fails to open, return a negative one.
     if((k3eeprog_log_file = fopen("k3eeprog.log","r")) == NULL)
	 return(-1);

     // Find the line in the log file that contain the register contents.
     // When it is found, determine the location of the PC register.
     while(fgets(linestring,80,k3eeprog_log_file) != NULL)
	  {
	   if(strstr(linestring,"PC") != NULL)
	     {
	      // If the PC address is $11B return a zero.
	      if(strstr(linestring,"11B") != NULL)
		{
		 // Find the memory display command.
		 fgets(linestring,80,k3eeprog_log_file);
		 fgets(linestring,80,k3eeprog_log_file);
		 fgets(linestring,80,k3eeprog_log_file);
		 strcpy(&linestring[0], &linestring[6]);

		 // Get the data from the log file.
		 for(i=0;i<16;i++)
		    {
		     data[i][2] = '\0';
		     strncpy(data[i],linestring,2);
		     strcpy(&linestring[0], &linestring[3]);
		    }

		 return(0);
		}

	      // If the PC address is any other address return a one.
	      else
		 return(1);
		}
	     }
	   return(1);
	  }
/*                      not_hex_number function
*
* Function input variables: char number;
*
* Function outputs: an integer;
*                   0: If the character is a hex number.
*                   1: If the character is not a hex number.
*
* This function determines if a character entered by the user is a valid hex
* number.
*
*/

int not_hex_number(char number)
     {

       // Return a one if the character is a valid hexadecimal number,
       // otherwise return a zero.
       if((('a'<= number) && (number <= 'f')) ||
	  (('0'<= number) && (number <='9')) ||
	  (('A'<= number) && (number <='F')))
	 return(0);
       else
	 return(1);
     }

/******************************************************************************
*                                                                             *
*                            MESSAGE BOX ROUTINES                             *
*                                                                             *
******************************************************************************/

/*                       main_screen function
*
* Function input variables: None.
*
* Function outputs: None.
*
* This function displays the program's opening screen.
*
*/

void main_screen(void)
     {
      int i; // counter variable

      // message box character array
      char text[7][80] ={"**************************************************************\n",
			 "*                                                            *\n",
			 "* Motorola MC68HC05K3 Personality EEPROM Programming Utility *\n",
			 "*                    Alpha Version 1.0                       *\n",
			 "*                    Copyright (c) 1995                      *\n",
			 "*                                                            *\n",
			 "**************************************************************\n"};

      // Clear screen.
      clrscr();

      // Display message box on the screen.
      for(i=0;i<7;i++)
	  print_center(i+9,text[i]);
      print_center(16,"PRESS ANY KEY TO CONTINUE");

      // Wait for any key to be pressed.
      while(!getch())
		;

      // Close the opening screen.
      clrscr();

      return;
     }

/*                       user_instruction_box function
*
* Function input variables: None.
*
* Function outputs: None.
*
* This function displays a box giving the user program operating instructions.
*
*/

void user_instruction_box(void)
     {
      int i; // counter variable

      // message box character array
      char text[6] [80] = {"************************ USER INSTRUCTIONS ****************************",
			   "*                                                                     *",
			   "* Connect your MMDS05 or EVS05 to a serial port of the host computer. *",
			   "*                Turn on your MMDS05 or EVS05.                        *",
			   "*                                                                     *",
			   "***********************************************************************"};

      // Clear screen.
       clrscr();

      // Display error box.
      for(i=0;i<6;i++)
	  print_center(i+9,text[i]);

      print_center(16,"PRESS ANY KEY TO CONTINUE");


      // Wait for the user to hit any key.
      while(!getch())
	    ;

      // Close error box.
      clrscr();

      return;
     }

/*                       data_verification_box function
*
* Function input variables: int mode.
*
* Function outputs: an integer;
*                   0: If the user data is correct.
*                   1: If the user data is incorrect and must be re-entered.
*
* This function displays a message box prompting the user to verify
* the data that is to be programmed into the device.
*
*/

int data_verification_box(int mode)
     {
      int i;  // generic counter variable
      char c;
      int return_flag; // function return value

      // message box character array
      char text[11][80] = {"******** YOU HAVE ENTERED THE FOLLOWING DATA: ********",
			   "** THE RESIDENT DEVICE CONTAINS THE FOLLOWING DATA: **",
			   "*                                                    *",
			   "*                                                    *",
			   "*                                                    *",
			   "*        Press <ENTER> if the data is correct.       *",
			   "*          Press <ESC> to abort this command.        *",
			   "*        Press any other key to re-enter data.       *",
			   "*      Press any key to return to the main menu.     *",
			   "*                                                    *",
			   "******************************************************"};

      char data_string[80];

      // Clear the screen.
      clrscr();

      // Display verification box.
      if(mode)
	 print_center(1,text[0]);
      else
	 print_center(1,text[1]);

      for(i=2;i<4;i++)
	  print_center(i,text[i]);

      sprintf(data_string,"*       Bits(0-7):     %s     Bits(64-71):    %s     *",
	      data[0], data[8]);
      print_center(4,data_string);
      sprintf(data_string,"*                                                    *");
      print_center(5,data_string);
      sprintf(data_string,"*       Bits(8-15):    %s     Bits(72-79):    %s     *",
	      data[1], data[9]);
      print_center(6,data_string);
      sprintf(data_string,"*                                                    *");
      print_center(7,data_string);
      sprintf(data_string,"*       Bits(16-23):   %s     Bits(80-87):    %s     *",
	      data[2], data[10]);
      print_center(8,data_string);
      sprintf(data_string,"*                                                    *");
      print_center(9,data_string);
      sprintf(data_string,"*       Bits(24-31):   %s     Bits(88-95):    %s     *",
	      data[3], data[11]);
      print_center(10,data_string);
      sprintf(data_string,"*                                                    *");
      print_center(11,data_string);
      sprintf(data_string,"*       Bits(32-39):   %s     Bits(96-103):   %s     *",
	      data[4], data[12]);
      print_center(12,data_string);
      sprintf(data_string,"*                                                    *");
      print_center(13,data_string);
      for(i=0;i<3;i++)
	 {
	  sprintf(data_string,"*       Bits(%d-%d):   %s     Bits(%d-%d):  %s     *",
		  (i*8)+40,(i*8)+47,data[i+5],(i*8)+104,(i*8)+111,data[i+13]);
	  print_center((i*2)+14,data_string);
	  sprintf(data_string,"*                                                    *");
	  print_center((i*2)+15,data_string);
	 }

      if(mode)
	{
	 for(i=5;i<8;i++)
	     print_center(i+15,text[i]);
	 for(i=9;i<11;i++)
	     print_center(i+14,text[i]);
	}
      else
	{
	 for(i=8;i<11;i++)
	     print_center(i+12,text[i]);
	}

      // Wait for a key to be pressed. If the <ENTER>key is pressed,
      // set the verified_data flag to zero.
      // Otherwise, return to the user data entry screen.
      while(!kbhit())
	    ;
      c = getch();

      if(mode)
	{
	 switch(c)
	    {
	     case 0x0D:
	       return_flag = 0;
	     break;
	     case 0x1B:
	       return_flag = 1;
	     break;
	     default:
	       return_flag = -1;
	     break;
	    }
	}

      // Close error box.
      clrscr();

      return(return_flag);
     }

/*                       programmed_box function
*
* Function input variables: None.
*
* Function outputs: None.
*
*
* This function displays a box informing the user that K3's personality
* EEPROM has been successfully programmed.
*
*/

void programmed_box(void)
     {
      int i; //counter variable

      // message box character array
      char text[5] [80] = {"*********************************************************************",
			   "*                                                                   *",
			   "* The resident MC69HC05K3's Personality EEPROM has been programmed. *",
			   "*                                                                   *",
			   "*********************************************************************"};

      // Clear screen and sound beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<5;i++)
	  print_center(i+9,text[i]);
      print_center(15,"PRESS ANY KEY TO EXIT TO DOS");

      // Wait for any key to be pressed.
      while(!getch())
	    ;

      // Close error message box.
      clrscr();

      return;
     }

/******************************************************************************
*                                                                             *
*                        ERROR INFORMATION BOX ROUTINES                       *
*                                                                             *
******************************************************************************/

/*                        invalid_choice function
*
* Function input variables: None.
*
* Function outputs: None.
*
* This function displays a error message box if the user enters an invalid
* entry in the command menu.
*/

void invalid_choice(void)
     {
      int i; // counter variable

      // message box character array
      char text[7][80] = {"*******************************",
			  "*                             *",
			  "*       Invalid choice.       *",
			  "*   Please enter 1, 2, or 3.  *",
			  "*                             *",
			  "*******************************",
			  "Press <RETURN> to continue."};

      // Clear the screen and beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<6;i++)
	  print_center(i+9,text[i]);

      print_center(16,text[6]);

      // Wait for the user to press the RETURN key.
      while((getch() != '\r'))
	     ;

      // Clear screen.
      clrscr();
      return;
     }
/*                           no_evs05 function
*
* Function input variables: None.
*
* Function outputs: None.
*
* This function displays an error box if EVS05.EXE is not found in the
* current directory.
*
*/

void no_evs05(void)
     {
      int i; // counter variable

      // message box character array
      char text[6][80] ={"******************** ERROR **************************",
			 "*                                                   *",
			 "* EVS05.EXE was not found in the current directory. *",
			 "*    This program requires EVS05.EXE to operate.    *",
			 "*                                                   *",
			 "*****************************************************"};

      // Clear the screen and sound a beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<6;i++)
	  print_center(i+9,text[i]);
      print_center(16,"PRESS ANY KEY TO EXIT TO DOS");

      // Wait for any key to be pressed.
      while(!getch())
	     ;

      // Close error box.
      clrscr();

      return;
     }

/*                           no_mmds05 function
*
* Function input variables: None.
*
* Function outputs: None.
*
* This function displays an error box if MMDS05.EXE is not found in the
* current directory.
*
*/

void no_mmds05(void)
     {
      int i; // counter variable

      // message box character array
      char text[6][80] = {"********************* ERROR **************************",
			  "*                                                    *",
			  "* MMDS05.EXE was not found in the current directory. *",
			  "*    This program requires MMDS05.EXE to operate.    *",
			  "*                                                    *",
			  "******************************************************"};

      // Clear the screen and sound a beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<6;i++)
	  print_center(i+9,text[i]);
      print_center(16,"PRESS <RETURN> TO EXIT TO DOS");

      // Wait for any key to be pressed.
      while(!getch())
	    ;

      // Close error message box.
      clrscr();

      return;
     }

/*                      no_personality_file function
*
* Function input variables: None.
*
* Function outputs: None.
*
* This function displays an error box if a K3 personality file
* found in the current directory.
*
*/

void no_personality_file(void)
     {
       int i; // counter variable

       // message box character array
       char text[6][80] = {"******************************** ERROR **************************************",
			   "*                                                                           *",
			   "* The device personality for the K3 was not found in the current directory. *",
			   "*       This program requires personality file 00014VXX.MEM to operate.     *",
			   "*                                                                           *",
			   "*****************************************************************************"};

      // Clear the screen and sound a beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<6;i++)
	  print_center(i+9,text[i]);
      print_center(16,"PRESS ANY KEY TO EXIT TO DOS");

      // Wait for the any key to be pressed.
      while(!getch())
	       ;

      // Close error message box.
      clrscr();

      return;
     }

/*                      emulator_error_box function
*
* Function input variables: None.
*
* Function outputs: None.
*
*
* This function displays an error box informing the user that an error has
* occurred in the operation of the emulator.
*
*/

void emulator_error_box(void)
     {
      int i; // counter variable

      // message box character array
      char text[5][80] = {"******************** ERROR ***************************",
			  "*                                                    *",
			  "* An error has occurred in the emulator's operation. *",
			  "*                                                    *",
			  "******************************************************"};

      // Clear the screen and sound a beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<5;i++)
	  print_center(i+9,text[i]);
      print_center(15,"PRESS <RETURN> TO EXIT TO DOS");

      // Wait for any key to be pressed.
      while(!getch())
	     ;

      // Close error box.
      clrscr();

      return;
     }

/*                      eeprom_error_box function
*
* Function input variables: None.
*
* Function outputs: None.
*
*
* This function displays an error box informing the user that the K3's
* personality EEPROM has failed to program.
*
*/

void eeprom_error_box(void)
     {
      int i; // counter variable

      // message box character array
      char text[5][80] = {"************************** ERROR **************************************",
			  "*                                                                     *",
			  "* The resident MC68HC05K3's Personality EEPROM has failed to program. *",
			  "*                                                                     *",
			  "***********************************************************************"};

      // Clear the screen and sound a beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<5;i++)
	  print_center(i+9,text[i]);
      print_center(15,"PRESS ANY KEY TO EXIT TO DOS");

      // Wait for any key to be pressed.
      while(!getch())
	    ;

      // Close error message box.
      clrscr();

      return;
     }

/*                    commandline_error_box function
*
* Function input variables: None
*
* Function outputs: None
*
* This function displays a screen informing the user that he/she has
* made an error in entering commandline parameters.
*
*/

void commandline_error_box(void)
     {
       int i; // counter variable

       // message box character array
       char text[11][80] = {"***************************** ERROR ********************************",
			    "*                                                                  *",
			    "*                                                                  *",
			    "*        Incorrect commandline parameters were entered.            *",
			    "*            Use the following syntax to envoke this program:      *",
			    "*               k3eeprog [Emulator Type] [COM Port].               *",
			    "* Type the following for the Emulator Type option: MMDS05 or EVS05 *",
			    "* Type the following for the Com Port option: 2 COM2 3 COM3 4 COM4 *",
			    "*                     (COM1 is the default)                        *",
			    "*                                                                  *",
			    "********************************************************************"};

       // Clear the screen and sound a beep.
       clrscr();
       beep();

       // Display error box.
       for(i=0;i<12;i++)
	   print_center(i+7,text[i]);
       print_center(18,"PRESS ANY KEY TO EXIT TO DOS");

       // Wait for the user to hit any key.
       while(!getch())
	 ;

       // Close the error box.
       clrscr();

       return;
     }

/*                      memory_error_box function
*
* Function input variables: None
*
* Function outputs: None
*
* This function displays a screen informing the user that he/she has
* made an error in entering commandline parameters.
*
*/

void memory_error_box(void)
     {
      int i; // counter variable

      // message box character array
      char text[5][80] = {"******************************************",
			  "*                                        *",
			  "* A memory allocation error has occurred *",
			  "*                                        *",
			  "******************************************"};

      // Clear the screen and sound a beep.
      clrscr();
      beep();

      // Display error message box.
      for(i=0;i<5;i++)
	  print_center(i+9,text[i]);
      print_center(15,"Press any key to exit to DOS.");

      // Wait for any key to be pressed.
      while(!getch())
	    ;

      return;
     }

/*                             beep function
*
* Function input variables: None.
*
* Function outputs: None.
*
* This function sounds a beep when an error is committed.
*
*/

void beep(void)
     {
      sound(500);
      delay(100);
      nosound();
      return;
     }


/*                           print_center function
*
* Function input variables: int y;
*                           vertical position at the string will be printed.
*                           char string[];
*                           string to be centered and printed on the screen.
*
* Function outputs: None.
*
* This function prints the character string passed to it in the center of the
* screen.
*
*/
void print_center(int y, char string[])
     {
      // Position the string in the center of the string.
      gotoxy (40 - (strlen(string)/2), y);

      // Print the string to the string.
      printf("%s",string);
     }
