/******************************************************************************
*									      *
*			       FUNCTION PROTOTYPES 			      *
*									      *
******************************************************************************/

// Constructs a startup file to program the resident MCU's PEEPROM.
int program_startup_file(void);

// Function calls the discrete functions that allow this program to read
// the resident MCU's PEEPROM.
void read_eeprom(char port[]);

// Constructs a startup file to read the resident MCU's PEEPROM.
int read_startup_file(void);

// Evaluates the results of a programming operation.
int evaluate_program_results(void);

// Evaluates the results of a read operation.
int evaluate_read_results(void);

// Allows the user to select the type of emulation system and COM port being
// used.
int system_setup(int commandline_number, char commandline_emulator[],
			char commandline_port[]);

// Allows the user to input the data to be programmed into the K3's PEEPROM.
void enter_data(void);

// Displays a box with user instructions.
void user_instruction_box(void);

// Displays an error box if MMDS05.EXE is not found in the current
// directory.
void no_mmds05(void);

// Displays an error box if the user enters an incorrect number or
// type of commandline parameters.
void commandline_error_box();

// Displays an error box if EVS05.EXE is not found in the current
// directory.
void no_evs05(void);

// Sounds a beep when an error is committed.
void beep(void);

// Displays an error box if the personality file for the K3 EM board is
// not found in the current directory.
void no_personality_file(void);

// Checks for MMDS05 or EVS05, and 00014VXX.MEM. These files are necessary for
// the program to operate.
int files_present(void);

// Displays K3EEPROG's opening screen.
void main_screen(void);

// Displays the program's main menu.
char user_selection(void);

// Calls the discrete functions that program the K3's Personality EEPROM.
void program_eeprom(char port[]);

// Displays a message box that asks for verification of the user entered data.
int data_verification_box(int mode);

// Displays an error box informing the user that an emulator error has occurred.
void emulator_error_box(void);

// Displays an error message box when the user enters an invalid menu choice.
void invalid_choice(void);

// Displays an error box informing the user that an error has occurred while
// programming the K3's Personality EEPROM.
void eeprom_error_box(void);

// Displays an error screen informing the user that a memory allocation error
// has occurred.
void memory_error_box(void);

// Displays a message box informing the user that the K3's Personality EEPROM
// has been successfully programmed with user data.
void programmed_box(void);

// Determines if a character entered by the user is a valid hexadecimal number.
int not_hex_number(char number);

// Prints a centered string on the screen.
void print_center(int y, char string[]);


// Embedded S-record PEEPROM programming algorithm.
char S19_data[]  [45] = {"S1130100A603B7083F0EA610B7D6160F1C0FA61EDF",
			 "S1130110AD481D0F170F3FD7BED7E6C0AD233CD760",
			 "S11301203AD626F44F3F0E5FA610B7D6390F463C99",
			 "S11301300E010FF8E1C026075C3AD626EF20FE2018",
			 "S1130140FE442410160F1A0FB7D8A60AAD0CB6D861",
			 "S11301501B0F170F3C0E010FE881AEC69D9D9D5AE3",
			 "S10D016026FB4A26F54FC703F08181",
			 "S10503FE0100F8",
			 "S9030000FC"};

// Embedded S-record PEEPROM reading algorithm.
char read_data[] [45] = {"S1130100A603B7083F0EA610B7D64F5F390F463C7B",
			 "S11001100E010FF8E7C05C3AD626F120FE80",
			 "S10503FE0100F8",
			 "S9030000FC"};





/******************************************************************************
*									      *
*			         GLOBAL VARIABLES 			      *
*									      *
******************************************************************************/
#define read 0
#define program 1

// Emulator type variable: 0 - MMDS05, 1 - EVS05.
int emulator_type;

// Data verification flag.
int verified_data = 1;

// User data array.
char data[16] [3];




